#!/bin/bash
# Copyright (C) 1999-2004,2007-2008 Red Hat, Inc. All rights reserved. This
# copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the
# GNU General Public License version 2.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# X Input method setup script

function is_dbus_enabled() {
    [ -n "$DBUS_SESSION_BUS_ADDRESS" ]
}

function is_imsettings_enabled() {
    [ -z "$DISABLE_IMSETTINGS" ] && is_dbus_enabled
}

function is_gtk_supported() {
    [ -n "$IMSETTINGS_DISABLE_DESKTOP_CHECK" ] && return 0
    case "$DESKTOP_SESSION" in
	*gnome|*kde|openbox|xfce4)
	    return 0
	    ;;
	*)
	    return 1
	    ;;
    esac
}

function is_qt_supported() {
    [ -n "$IMSETTINGS_DISABLE_DESKTOP_CHECK" ] && return 0
    case "$DESKTOP_SESSION" in
	*)
	    return 1
	    ;;
    esac
}

function setup_gtk_immodule() {
    if is_imsettings_enabled && is_gtk_supported; then
	# Ensure GTK_IM_MODULE is empty. otherwise GTK+ doesn't pick up immodule through XSETTINGS
	unset GTK_IM_MODULE
	export GTK_IM_MODULE
    else
	[ -n "$GTK_IM_MODULE" ] && export GTK_IM_MODULE
    fi
}

function setup_qt_immodule() {
    if is_imsettings_enabled && is_qt_supported; then
	# FIXME: Qt doesn't support XSETTINGS for immodule yet.
	#        We still need to go with the older way.
	[ -n "$QT_IM_MODULE" ] && export QT_IM_MODULE
    else
	[ -n "$QT_IM_MODULE" ] && export QT_IM_MODULE
    fi
}

function setup_xim() {
    if is_imsettings_enabled; then
	# setup XMODIFIERS
	XMODIFIERS="@im=imsettings"
	export XMODIFIERS
    else
	[ -z "$XMODIFIERS" -a -n "$XIM" ] && XMODIFIERS="@im=$XIM"
	[ -n "$XMODIFIERS" ] && export XMODIFIERS
    fi
}

function run_imsettings() {
    print_info
    if [ -n "$DRY_RUN" ]; then
	echo "*** DRY RUN MODE: running IM through imsettings"
    else
	which imsettings-start > /dev/null 2>&1 && LANG="$tmplang" imsettings-start -n "$IMSETTINGS_MODULE" || :
	which imsettings-xim > /dev/null 2>&1 && imsettings-xim &
    fi
}

function run_xim() {
    print_info
    if [ -n "$DRY_RUN" ]; then
	echo "*** DRY RUN MODE: running IM without imsettings"
    else
	DISABLE_IMSETTINGS=true
	export DISABLE_IMSETTINGS

	# execute XIM_PROGRAM
	[ -n "$XIM_PROGRAM" ] && which "$XIM_PROGRAM" > /dev/null 2>&1 && LANG="$tmplang" "$XIM_PROGRAM" $XIM_ARGS &
    fi
}

function print_result() {
    $1
    if [ $? -eq 0 ]; then
	echo yes
    else
	echo no
    fi
}

function print_info() {
    echo "imsettings information"
    echo "=========================="
    echo -n "Is DBus enabled: "
    print_result is_dbus_enabled
    echo -n "Is imsettings enabled: "
    print_result is_imsettings_enabled
    echo -n "Is GTK+ supported: "
    print_result is_gtk_supported
    echo -n "Is Qt supported: "
    print_result is_qt_supported
    echo "DESKTOP_SESSION: $DESKTOP_SESSION"
    echo "DISABLE_IMSETTINGS: $DISABLE_IMSETTINGS"
    echo "IMSETTINGS_DISABLE_DESKTOP_CHECK: $IMSETTINGS_DISABLE_DESKTOP_CHECK"
    echo "DBUS_SESSION_BUS_ADDRESS: $DBUS_SESSION_BUS_ADDRESS"
    echo "GTK_IM_MODULE: $GTK_IM_MODULE"
    echo "QT_IM_MODULE: $QT_IM_MODULE"
    echo "XMODIFIERS: $XMODIFIERS"
    echo "IMSETTINGS_MODULE: $IMSETTINGS_MODULE"
    echo ""
}

USER_XINPUTRC="$HOME/.xinputrc"
SYS_XINPUTRC="/etc/X11/xinit//xinputrc"

# Load up the user and system locale settings
oldterm=$TERM
unset TERM
if [ -r /etc/profile.d/lang.sh ]; then
    # for Fedora etc
    source /etc/profile.d/lang.sh
elif [ -r /etc/default/locale ]; then
    # for Debian
    source /etc/default/locale
fi
[ -n "$oldterm" ] && export TERM=$oldterm

tmplang=${LC_CTYPE:-${LANG:-"en_US.UTF-8"}}

# unset env vars to be safe
unset AUXILIARY_PROGRAM AUXILIARY_ARGS GTK_IM_MODULE ICON IMSETTINGS_IGNORE_ME LONG_DESC PREFERENCE_PROGRAM PREFERENCE_ARGS QT_IM_MODULE SHORT_DESC XIM XIM_PROGRAM XIM_ARGS XMODIFIERS

[ -z "$IMSETTINGS_DISABLE_USER_XINPUTRC" ] && IMSETTINGS_DISABLE_USER_XINPUTRC=no

if [ -r "$USER_XINPUTRC" -a "x$IMSETTINGS_DISABLE_USER_XINPUTRC" = "xno" ]; then
    source "$USER_XINPUTRC"
    if [ ! -h "$USER_XINPUTRC" ]; then
	SHORT_DESC="User Specific"
    fi
elif [ -r "$SYS_XINPUTRC" ]; then
    # FIXME: This hardcoded list has to be gone in the future.
    _language_list="as bn gu hi ja kn ko ml mr ne or pa si ta te th ur vi zh"
    _sourced_xinputrc=0
    for i in $_language_list; do
        if echo $tmplang | grep -q -E "^$i"; then
            source "$SYS_XINPUTRC"
            _sourced_xinputrc=1
            break
        fi
    done
    if [ $_sourced_xinputrc -eq 0 ]; then
        # Read none.conf to set up properly for locales not listed the above.
        source /etc/X11/xinit/xinput.d//none.conf
    fi
fi

[ -z "$XIM" ] && XIM=none

# start IM via imsettings
IMSETTINGS_MODULE=${SHORT_DESC:-${XIM}}
[ -z "$IMSETTINGS_MODULE" ] && IMSETTINGS_MODULE="none"

# NOTE: Please make sure the session bus is established before running this script.
if ! is_dbus_enabled; then
    echo "***"
    echo "*** No DBus session hasn't been established yet. giving up to deal with Input Method with imsettings."
    echo "***"

    setup_gtk_immodule
    setup_qt_immodule
    setup_xim

    run_xim
elif ! is_imsettings_enabled; then
    echo "***"
    echo "*** imsettings is explicitly disabled."
    echo "***"

    setup_gtk_immodule
    setup_qt_immodule
    setup_xim

    run_xim
else
    # Yes, we are in the dbus session
    setup_gtk_immodule
    setup_qt_immodule
    setup_xim

    run_imsettings
fi
